unit ControllerUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, Dewesoft_TLB, ComObj;

type
  TController = class;

  TEventSink = class(TInterfacedObject, IUnknown, IDispatch)
  private
    Controller: TController;

    {IUknown methods}
    function QueryInterface(const IID: TGUID; out Obj):HResult;stdcall;
    {Idispatch}
    function GetTypeInfoCount(out Count: Integer): HResult; stdcall;
    function GetTypeInfo(Index, LocaleID: Integer; out TypeInfo): HResult; stdcall;
    function GetIDsOfNames(const IID: TGUID; Names: Pointer;
      NameCount, LocaleID: Integer; DispIDs: Pointer): HResult; stdcall;
    function Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer;
      Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult; stdcall;
  end;

  TController = class(TComponent)
  private
    FOnTestStarted, FOnGetData, FOnTestFinished: TNotifyEvent;
    FEvents: TEventSink;
    FConnectionToken: Integer;
  public
    Started: Boolean;
    Success:Boolean;
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
    procedure StartTest;
    procedure DeweGetData;
    procedure StopTest;

    property OnTestStarted:TNotifyEvent read FOnTestStarted write FOnTestStarted;
    property OnGetData:TNotifyEvent read FOnGetData write FOnGetData;
    property OnTestFinished:TNotifyEvent read FOnTestFinished write FOnTestFinished;
  end;

var
  DeweApp: App;

implementation

uses
  Registry, ActiveX;

constructor TController.Create(AOwner: TComponent);
begin
  try
    inherited Create(AOwner);

    OutputDebugString('Start App...');
    DeweApp := CoApp.Create;
    DeweApp.Init;
    DeweApp.ShowPropertyFrame := True;
    DeweApp.ShowStoreOptions := True;
    DeweApp.ShowSROptions := True;
    DeweApp.ShowCaptionBar;

    // this and also DeweEvents is just to read OnGetData event - it is always fixed
    FEvents := TEventSink.Create;
    FEvents.Controller := Self;
    InterfaceConnect(DeweApp, IAppEvents, FEvents, FConnectionToken);
  except on E:Exception do
    OutputDebugString(PWideChar('Exception while opening dewesoft: ' + E.Message));
  end;
end;

destructor TController.Destroy;
begin
  try
    if DeweApp <> nil then begin
      if Started then StopTest;
      if DeweApp.IsAcqRunning then DeweApp.Stop;
    end;
    Sleep(1000);

    InterfaceDisconnect(DeweApp, IAppEvents, FConnectionToken);

    DeweApp := nil;
    FEvents := nil;
    inherited;
  except
    Windows.MessageBox(null, 'Error occured while closing DEWESoft.'#13#10'You should first close the Microsoft Exception (Dont send button) before closing this message!', 'DEWESoft Close Error', MB_SETFOREGROUND or MB_TOPMOST or MB_ICONHAND);
  end;
end;

procedure TController.StartTest;
begin
  try
    Started := True;
    if Assigned(FOnTestStarted) then FOnTestStarted(Self);
  except on E:Exception do
    OutputDebugString('Error starttest');
  end;
end;

procedure TController.DeweGetData;
begin
  if Assigned(OnGetData) then
    OnGetData(Self);
end;

procedure TController.StopTest;
begin
  try
    Started := False;
    if DeweApp.IsAcqRunning then DeweApp.Stop;
    if Assigned(FOnTestFinished) then FOnTestFinished(Self);
  except on E:Exception do
    OutputDebugString('Controller.StopTest');
  end;
end;

{ TEventSink }

function TEventSink.Invoke(DispID: integer; const IID: TGUID; LocaleID: integer; Flags: Word; var Params; VarResult,ExcepInfo,ArgErr:Pointer): HResult;
begin
  Result := S_OK;

  case DispID of
    201: Controller.DeweGetData;
  end;
end;

function TEventSink.QueryInterface(const IID: TGUID; out Obj):HResult;stdcall;
begin
  if GetInterFace(IID,Obj) then
    Result := S_OK
  else if IsEqualIID(IID, IAppEvents) then
    Result := QueryInterface(IDispatch,Obj)
  else
    Result := E_NOINTERFACE;
end;

function TEventSink.GetTypeInfoCount(out Count: Integer): HResult;
begin
  Result := S_OK;
end;

function TEventSink.GetTypeInfo(Index, LocaleID: Integer; out TypeInfo): HResult;
begin
  Result := S_OK;
end;

function TEventSink.GetIDsOfNames(const IID: TGUID; Names: Pointer;
      NameCount, LocaleID: Integer; DispIDs: Pointer): HResult;
begin
  Result := S_OK;
end;

end.
