% this function is called for each event from Dewesoft
function eventhandler(varargin)

% global variables from DWAutomationExample.m
global actx;
global start_calc;
global ReadCh;
global WriteCh;
persistent OldSamples;

% get incoming event args
eventargs = varargin{end-1};

% check which event was fired - in this example we handle only OnGetData
if (strcmp(eventargs.Type, 'OnGetData'))   
   
    % we will read blocks of size BlockSize
    BlockSize = 128;
    if start_calc == 0 
        OldSamples = 0;
        start_calc = 1;
    end
   
    % we are going to use the first sync channel as a reference channel:
    % reference channel will be used for retrieving number of sync samples
    % received during the measurement and calculating timestamps
    RefCh = actx.Data.AllChannels.Item(0);
   
    % in the current OnGetData call we will read only remaining samples i.e.
    % number of all samples received so far (RefCh.DBPos) minus number of samples
    % already read in previous OnGetData calls
    NewSamples = RefCh.DBPos - OldSamples; 
    if NewSamples < 0 
        NewSamples = NewSamples + RefCh.DBBufSize; % ring buffer "wrap around" case
    end
    
    % we are going to read new samples in blocks of size BlockSize
    BlockCountToRead = floor(NewSamples / BlockSize);

    % read data (in blocks) from ReadCh, transform it and write it to WriteCh
    for i = 0 : BlockCountToRead - 1
        
        % read one data block at the time
        % GetScaledDataEx(StartSamples, Count);
        DataBlock = ReadCh.GetScaledDataEx(ReadCh.DBDataSize - NewSamples + i * BlockSize, BlockSize);
        subplot(2,1,1);
        plot(DataBlock); 
    end
 
    % update sample counter
    OldSamples = RefCh.DbPos - NewSamples + BlockCountToRead * BlockSize;
    if OldSamples < 0
        OldSamples = OldSamples + RefCh.DBBufSize; 
    end
end
